@@ -3,8 +3,8 @@
 #include "libfastmint.h"
 
 #if (defined(__i386__) || defined(__AMD64__)) && defined(__GNUC__) && defined(__MMX__)
-typedef int mmx_d_t __attribute__ ((mode(V2SI)));
-typedef int mmx_q_t __attribute__ ((mode(DI)));
+typedef int mmx_d_t __attribute__ ((vector_size (8)));
+typedef int mmx_q_t __attribute__ ((vector_size (8)));
 #endif
 
 int minter_mmx_standard_1_test(void)
@@ -314,7 +314,9 @@ unsigned long minter_mmx_standard_1(int bits, int* best, unsigned char *block, c
 	
   /* Work out which bits to mask out for test */
   if(maxBits < 32) {
-    bitMask1Low = ~((((uInt32) 1) << (32 - maxBits)) - 1);
+    if ( bits == 0 ) { bitMask1Low = 0; } else {
+      bitMask1Low = ~((((uInt32) 1) << (32 - maxBits)) - 1);
+    }
     bitMask1High = 0;
   } else {
     bitMask1Low = ~0;
