@@ -282,7 +282,7 @@ unsigned long hashcash_per_sec_calc( void )
 		begin = end;
 		do {
 			/* set up SHA-1 block */
-			strncpy(block, test_string, SHA1_INPUT_BYTES);
+			strncpy((char*)block, test_string, SHA1_INPUT_BYTES);
 			block[test_tail] = 0x80;
 			memset(block+test_tail+1, 0, 59-test_tail);
 			PUT_WORD(block+60, test_tail << 3);
@@ -312,7 +312,7 @@ unsigned long hashcash_per_sec_calc( void )
 	 * elapsed time for precision */
 	while(iter_count) {
 		/* set up SHA-1 block */
-		strncpy(block, test_string, SHA1_INPUT_BYTES);
+		strncpy((char*)block, test_string, SHA1_INPUT_BYTES);
 		block[test_tail] = 0x80;
 		memset(block+test_tail+1, 0, 59-test_tail);
 		PUT_WORD(block+60, test_tail << 3);
@@ -410,7 +410,7 @@ unsigned long hashcash_benchtest(int verbose, int core)
 		}
 		
 		/* set up SHA-1 block */
-		strncpy(block, test_string, SHA1_INPUT_BYTES);
+		strncpy((char*)block, test_string, SHA1_INPUT_BYTES);
 		block[test_tail] = 0x80;
 		memset(block+test_tail+1, 0, 59-test_tail);
 		PUT_WORD(block+60, test_tail << 3);
@@ -548,7 +548,7 @@ double hashcash_fastmint(const int bits, const char *token, int compress,
 	unsigned int IV[SHA1_DIGEST_WORDS] = {0};
 	unsigned char *buffer = NULL, *block = NULL, c = 0;
 	unsigned int buflen = 0, tail = 0, a = 0, b = 0, save_tail = 0;
-	unsigned long t = 0, loop = 0, iters = 0, i = 0;
+	unsigned long t = 0, loop = 0, iters = 0, i = 0, first = 1;
 	HC_Mint_Routine best_minter;
 	double counter = 0, expected = 0;
 	/* this is to allow this fn to call the same callback macro */
@@ -574,7 +574,7 @@ double hashcash_fastmint(const int bits, const char *token, int compress,
 	buflen = (tail - (tail % SHA1_INPUT_BYTES)) + 2*SHA1_INPUT_BYTES;
 	buffer = malloc(buflen);
 	memset(buffer, 0, buflen);
-	strncpy(buffer, token, buflen);
+	strncpy((char*)buffer, token, buflen);
 	
 	/* Add 96 bits of random data */
 	t = tail + 16;
@@ -589,7 +589,9 @@ double hashcash_fastmint(const int bits, const char *token, int compress,
 
 	bit_rate = EncodeBitRate[encodings[fastest_minter]];
 	chars = 31/bit_rate;
-	for ( i = compress ? 1 : chars; i <= chars && gotBits < bits; i++ ) {
+	for ( i = compress ? 1 : chars; 
+	      i <= chars && (first || gotBits < bits); i++ ) {
+		first = 0;
 		tail = save_tail;
 		t = tail + i;
 		for( ; tail < t; tail++) { buffer[tail] = '0'; }
